@@ -21,8 +21,10 @@
 #include "panel-typebuiltins.h"
 #include "panel-applet-frame.h"
 #include "panel-globals.h"
+#include "panel-profile.h"
+#include "panel-lockdown.h"
 
-#define MOVE_INCREMENT 2
+#define MOVE_INCREMENT 1
 
 typedef enum {
 	PANEL_SWITCH_MOVE = 0,
@@ -32,6 +34,7 @@ typedef enum {
 
 enum {
 	SIZE_CHANGE_SIGNAL,
+	BACK_CHANGE_SIGNAL,
 	APPLET_MOVE_SIGNAL,
 	APPLET_ADDED_SIGNAL,
 	APPLET_REMOVED_SIGNAL,
@@ -71,6 +74,9 @@ static void panel_widget_state_changed  (GtkWidget        *widget,
 static void panel_widget_style_set      (GtkWidget        *widget,
 					 GtkStyle         *previous_style);
 
+static void panel_widget_background_changed (PanelBackground *background,
+					     PanelWidget     *panel);
+
 static void panel_widget_push_move_applet   (PanelWidget      *panel,
                                              GtkDirectionType  dir);
 static void panel_widget_switch_move_applet (PanelWidget      *panel,
@@ -90,33 +96,6 @@ static gboolean panel_widget_push_applet_left  (PanelWidget *panel,
 						GList       *list,
 						int          push);
 
-/************************
- debugging
- ************************/
-
-#ifdef PANEL_WIDGET_DEBUG
-static void
-debug_dump_panel_list(PanelWidget *panel)
-{
-	GList *list;
-	puts("\nDUMP START\n");
-	for(list = panel->applet_list;list!=NULL;list=g_list_next(list)) {
-		AppletData *ad = list->data;
-		printf("pos: %d cells: %d\n",ad->pos,ad->cells);
-	}
-	puts("\nDUMP END\n");
-}
-
-static void
-debug_panel_widget_dump_applet_data (AppletData *data) 
-{
-	printf ("===== Applet Dumped =====\n");
-	printf ("\tPosition : %d\n", data->pos);
-	printf ("\tCells : %d\n", data->cells);
-	printf ("\tDrag Off: %d\n", data->drag_off);
-}
-#endif
-
 /************************
  convenience functions
  ************************/
@@ -336,6 +315,17 @@ panel_widget_class_init (PanelWidgetClass *class)
                               G_TYPE_NONE,
                               0);
 
+	panel_widget_signals[BACK_CHANGE_SIGNAL] =
+                g_signal_new ("back_change",
+                              G_TYPE_FROM_CLASS (object_class),
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET (PanelWidgetClass, back_change),
+                              NULL,
+                              NULL, 
+                              panel_marshal_VOID__VOID,
+                              G_TYPE_NONE,
+                              0);
+
 	panel_widget_signals[APPLET_MOVE_SIGNAL] =
                 g_signal_new ("applet_move",
                               G_TYPE_FROM_CLASS (object_class),
@@ -432,6 +422,7 @@ panel_widget_class_init (PanelWidgetClass *class)
                               0);
 
 	class->size_change = NULL;
+	class->back_change = NULL;
 	class->applet_move = NULL;
 	class->applet_added = NULL;
 	class->applet_removed = NULL;
@@ -1709,7 +1700,9 @@ panel_widget_instance_init (PanelWidget *panel)
 	panel->drop_widget   = widget;
 	panel->open_dialogs  = NULL;
 
-	panel_background_init (&panel->background);
+	panel_background_init (&panel->background,
+			       (PanelBackgroundChangedNotify) panel_widget_background_changed,
+			       panel);
 
 	panels = g_slist_append (panels, panel);
 }
@@ -2085,7 +2078,8 @@ panel_widget_applet_move_to_cursor (PanelWidget *panel)
 			    panel_widget_is_cursor (new_panel,10) &&
 			    panel_screen_from_panel_widget (panel) ==
 			    panel_screen_from_panel_widget (new_panel) &&
-			    !g_slist_find (forb, new_panel)) {
+			    !g_slist_find (forb, new_panel) &&
+			    !panel_lockdown_get_locked_down ()) {
 				pos = panel_widget_get_moveby (new_panel, 0, ad->drag_off);
 
 				if (pos < 0) pos = 0;
@@ -2192,75 +2186,94 @@ schedule_try_move(PanelWidget *panel, gboolean repeater)
 }
 
 static gboolean
-panel_widget_applet_event(GtkWidget *widget, GdkEvent *event)
+panel_widget_applet_button_press_event (GtkWidget      *widget,
+					GdkEventButton *event)
 {
 	PanelWidget *panel;
-	GdkEventButton *bevent;
+	guint32      event_time;
+	
+	g_return_val_if_fail (PANEL_IS_WIDGET (widget->parent), FALSE);
 
-	g_return_val_if_fail(GTK_IS_WIDGET(widget),FALSE);
-	g_return_val_if_fail(PANEL_IS_WIDGET(widget->parent),FALSE);
-	g_return_val_if_fail(event!=NULL,FALSE);
+	panel = PANEL_WIDGET (widget->parent);
 
-	panel = PANEL_WIDGET(widget->parent);
-
-	switch (event->type) {
-		case GDK_BUTTON_PRESS:
-			bevent = (GdkEventButton *) event;
-#ifdef PANEL_WIDGET_DEBUG
-			printf("the appwidget %lX\n",(long)widget);
-#endif
-
-			/* don't propagate this event */
-			if (panel->currently_dragged_applet) {
-				g_signal_stop_emission 
-					(G_OBJECT (widget), 
-					 g_signal_lookup ("event", G_OBJECT_TYPE (widget)),
-					 0);
-				return TRUE;
-			}
-
-			if ( ! commie_mode &&
-			    bevent->button == 2) {
-				guint32 time_ = bevent->time;
-				/* time on sent events seems to be bogus */
-				if (bevent->send_event)
-					time_ = GDK_CURRENT_TIME;
-				/* Start drag */
-				panel_widget_applet_drag_start (
-					panel, widget, PW_DRAG_OFF_CURSOR, time_);
-				return TRUE;
-			}
-
-			break;
-
-		case GDK_BUTTON_RELEASE:
-			if (panel->currently_dragged_applet) {
-				g_signal_stop_emission
-					(G_OBJECT (widget),
-					g_signal_lookup ("event", G_OBJECT_TYPE (widget)),
+	/* don't propagate this event */
+	if (panel->currently_dragged_applet) {
+		g_signal_stop_emission (G_OBJECT (widget), 
+					g_signal_lookup ("button-press-event",
+							 G_OBJECT_TYPE (widget)),
 					0);
-				panel_widget_applet_drag_end(panel);
-				return TRUE;
-			}
+		return TRUE;
+	}
 
-			break;
-		case GDK_MOTION_NOTIFY:
-			schedule_try_move(panel, FALSE);
-			break;
-		case GDK_KEY_PRESS:
-			if (panel_applet_in_drag) {
-				return gtk_bindings_activate (GTK_OBJECT (panel),
-					((GdkEventKey *)event)->keyval, 
-					((GdkEventKey *)event)->state);	
-			}
-			break;
-		default:
-			break;
+	if (panel_lockdown_get_locked_down () || event->button != 2)
+		return FALSE;
+
+	/* time on sent events seems to be bogus */
+	event_time = event->time;
+	if (event->send_event)
+		event_time = GDK_CURRENT_TIME;
+
+	panel_widget_applet_drag_start (panel, widget, PW_DRAG_OFF_CURSOR, event_time);
+
+	return TRUE;
+}
+
+static gboolean
+panel_widget_applet_button_release_event (GtkWidget      *widget,
+					  GdkEventButton *event)
+{
+	PanelWidget *panel;
+	
+	g_return_val_if_fail (PANEL_IS_WIDGET (widget->parent), FALSE);
+
+	panel = PANEL_WIDGET (widget->parent);
+	
+	/* don't propagate this event */
+	if (panel->currently_dragged_applet) {
+		g_signal_stop_emission (G_OBJECT (widget),
+					g_signal_lookup ("button-release-event",
+							 G_OBJECT_TYPE (widget)),
+					0);
+		panel_widget_applet_drag_end (panel);
+		return TRUE;
 	}
 
 	return FALSE;
 }
 
+static gboolean
+panel_widget_applet_motion_notify_event (GtkWidget      *widget,
+					 GdkEventMotion *event)
+{
+	PanelWidget *panel;
+	
+	g_return_val_if_fail (PANEL_IS_WIDGET (widget->parent), FALSE);
+
+	panel = PANEL_WIDGET (widget->parent);
+	
+	schedule_try_move (panel, FALSE);
+
+	return FALSE;
+}
+
+static gboolean
+panel_widget_applet_key_press_event (GtkWidget   *widget,
+				     GdkEventKey *event)
+{
+	PanelWidget *panel;
+	
+	g_return_val_if_fail (PANEL_IS_WIDGET (widget->parent), FALSE);
+
+	panel = PANEL_WIDGET (widget->parent);
+
+	if (!panel_applet_in_drag)
+		return FALSE;
+	
+	return gtk_bindings_activate (GTK_OBJECT (panel),
+				      ((GdkEventKey *)event)->keyval, 
+				      ((GdkEventKey *)event)->state);	
+}
+
 static int
 panel_sub_event_handler(GtkWidget *widget, GdkEvent *event, gpointer data)
 {
@@ -2338,7 +2351,6 @@ panel_widget_applet_destroy (GtkWidget *applet, gpointer data)
 	g_free (ad);
 }
 
-
 static void
 bind_top_applet_events (GtkWidget *widget)
 {
@@ -2348,9 +2360,19 @@ bind_top_applet_events (GtkWidget *widget)
 			  G_CALLBACK (panel_widget_applet_destroy),
 			  NULL);
 
-	g_signal_connect (G_OBJECT(widget),
-			  "event",
-			  G_CALLBACK (panel_widget_applet_event),
+	/* connect-after so that we stop the propogation of the event */
+	g_signal_connect_after (widget, "button-press-event",
+				G_CALLBACK (panel_widget_applet_button_press_event),
+				NULL);
+
+	g_signal_connect (widget, "button-release-event",
+			  G_CALLBACK (panel_widget_applet_button_release_event),
+			  NULL);
+	g_signal_connect (widget, "motion-notify-event",
+			  G_CALLBACK (panel_widget_applet_motion_notify_event),
+			  NULL);
+	g_signal_connect (widget, "key-press-event",
+			  G_CALLBACK (panel_widget_applet_key_press_event),
 			  NULL);
 
 	/* XXX: This is more or less a hack.  We need to be able to
@@ -2610,6 +2632,16 @@ panel_widget_set_size (PanelWidget *panel_widget,
 	gtk_widget_queue_resize (GTK_WIDGET (panel_widget));
 }
 
+static void
+panel_widget_background_changed (PanelBackground *background,
+				 PanelWidget     *panel)
+{
+	g_return_if_fail (PANEL_IS_WIDGET (panel));
+	g_signal_emit (G_OBJECT (panel),
+		       panel_widget_signals [BACK_CHANGE_SIGNAL],
+		       0);
+}
+
 static void 
 panel_widget_push_move_applet (PanelWidget     *panel,
                                GtkDirectionType dir)
@@ -2731,7 +2763,9 @@ panel_widget_tab_move (PanelWidget *panel,
 	if (!new_panel && previous_panel)
 		new_panel = previous_panel;
 	
-	if (new_panel && (new_panel != panel))
+	if (new_panel &&
+	    (new_panel != panel) &&
+	    !panel_lockdown_get_locked_down ())
 		panel_widget_reparent (panel, new_panel, ad->applet, 0);
 }
 
@@ -2896,7 +2930,8 @@ panel_widget_register_open_dialog (PanelWidget *panel,
 	panel->open_dialogs = g_slist_append (panel->open_dialogs,
 					      dialog);
 	
-	panel_signal_connect_object_while_alive (dialog, "destroy",
-	 		 G_CALLBACK (panel_widget_open_dialog_destroyed),
-			 panel);
+	g_signal_connect_object (dialog, "destroy",
+				 G_CALLBACK (panel_widget_open_dialog_destroyed),
+				 panel,
+				 G_CONNECT_SWAPPED);
 }
